@@ -1,6 +1,6 @@
 /* Mailnag - GNOME-Shell extension frontend
 *
-* Copyright 2013 - 2016 Patrick Ulbrich <zulu99@gmx.net>
+* Copyright 2013 - 2021 Patrick Ulbrich <zulu99@gmx.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 const Main = imports.ui.main;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
-const Lang = imports.lang;
 const ExtensionUtils = imports.misc.extensionUtils;
 const Me = ExtensionUtils.getCurrentExtension();
 const Convenience = Me.imports.convenience;
@@ -36,13 +35,15 @@ const SHOW_DATES_KEY					= 'show-dates';
 const SHOW_MARK_ALL_AS_READ_BUTTON_KEY	= 'show-mark-all-as-read-button';
 const SHOW_CHECK_FOR_MAIL_BUTTON_KEY	= 'show-check-for-mail-button';
 const SHOW_SETTINGS_BUTTON_KEY			= 'show-settings-button';
+const SHOW_QUIT_BUTTON_KEY				= 'show-quit-button';
 const GROUP_BY_ACCOUNT_KEY				= 'group-by-account';
 const REMOVE_INDICATOR_KEY				= 'remove-indicator';
 
 const KeyActionMap = new Map([
 	[ SHOW_MARK_ALL_AS_READ_BUTTON_KEY,	Opts.ACTION_FLAGS.MARK_ALL_AS_READ ],
 	[ SHOW_CHECK_FOR_MAIL_BUTTON_KEY,	Opts.ACTION_FLAGS.CHECK_FOR_MAIL ],
-	[ SHOW_SETTINGS_BUTTON_KEY,			Opts.ACTION_FLAGS.SETTINGS ]
+	[ SHOW_SETTINGS_BUTTON_KEY,			Opts.ACTION_FLAGS.SETTINGS ],
+	[ SHOW_QUIT_BUTTON_KEY,				Opts.ACTION_FLAGS.QUIT ]
 ]);
 
 const MailnagIface =
@@ -71,45 +72,41 @@ const MailnagIface =
 
 const MailnagDbus = Gio.DBusProxy.makeProxyWrapper(MailnagIface);
 
-const MailnagExtension = new Lang.Class({
-	Name: 'MailnagExtension',
-	
-	_init: function(options) {
+var MailnagExtension = class {
+	constructor(options) {
 		
 		this._mails = [];
 		this._opts = options;
 		this._indicator = null;
 		
-		this._proxy = new MailnagDbus(Gio.DBus.session,
-			'mailnag.MailnagService', '/mailnag/MailnagService');
+		var proxy_ready_cb = function() {
+			this._onMailsAddedId = this._proxy.connectSignal('MailsAdded', this._onMailsAdded.bind(this));
+			this._onMailsRemovedId = this._proxy.connectSignal('MailsRemoved', this._onMailsRemoved.bind(this));
 			
-		this._onMailsAddedId = this._proxy.connectSignal('MailsAdded',
-			Lang.bind(this, this._onMailsAdded));
-		
-		this._onMailsRemovedId = this._proxy.connectSignal('MailsRemoved',
-			Lang.bind(this, this._onMailsRemoved));
-		
-		// TODO : what if Mailnag sends a MailsAdded signal here or after GetMailsRemote()
-		// (should happen *extremely* rarely)?
-		// Is it possible to prevent the extension from notifying twice?
-		
-		// Mailnag possibly fired a 'MailsAdded' signal before this extension was started,
-		// so check if Mailnag fetched mails already and pull them manually.
-		this._proxy.GetMailsRemote(Lang.bind(this,
-			function([mails], error) {
-				if (!error) {
-					if ((mails.length > 0) || !this._opts.removeIndicator) {
-						this._handle_new_mails(mails, mails);
+			// TODO : what if Mailnag sends a MailsAdded signal here or after GetMailsRemote()
+			// (should happen *extremely* rarely)?
+			// Is it possible to prevent the extension from notifying twice?
+			
+			// Mailnag possibly fired a 'MailsAdded' signal before this extension was started,
+			// so check if Mailnag fetched mails already and pull them manually.
+			this._proxy.GetMailsRemote(([mails], error) => {
+					if (!error) {
+						if ((mails.length > 0) || !this._opts.removeIndicator) {
+							this._handle_new_mails(mails, mails);
+						}
 					}
-				}
-			}));
-	},
+				});
+		};
+		
+		this._proxy = new MailnagDbus(Gio.DBus.session,
+			'mailnag.MailnagService', '/mailnag/MailnagService', proxy_ready_cb.bind(this));
+	}
 	
-	_onMailsAdded: function(proxy, sender, [new_mails, all_mails]) {
+	_onMailsAdded(proxy, sender, [new_mails, all_mails]) {
 		this._handle_new_mails(new_mails, all_mails);
-	},
+	}
 	
-	_onMailsRemoved: function(proxy, sender, [remaining_mails]) {
+	_onMailsRemoved(proxy, sender, [remaining_mails]) {
 		// TODO : not only support removal of *all* mails
 		if (remaining_mails.length == 0) {
 			this._mails = [];
@@ -122,9 +119,9 @@ const MailnagExtension = new Lang.Class({
 				}
 			}
 		}
-	},
+	}
 	
-	_handle_new_mails: function(new_mails, all_mails) {
+	_handle_new_mails(new_mails, all_mails) {
 		this._mails = all_mails;
 		
 		if (this._indicator == null) {
@@ -132,26 +129,26 @@ const MailnagExtension = new Lang.Class({
 		} else {
 			this._indicator.setMails(all_mails);
 		}
-	},
+	}
 	
-	_createIndicator: function() {
+	_createIndicator() {
 		this._indicator = new Indicator.MailnagIndicator(this._opts, this);
 		this._indicator.setMails(this._mails);
 		Main.panel.addToStatusArea('mailnag-indicator', this._indicator, 0);
-	},
+	}
 	
-	_destroyIndicator: function() {
+	_destroyIndicator() {
 		if (this._indicator != null) {
 			this._indicator.destroy();
 			this._indicator = null;
 		}
-	},
+	}
 	
-	markMailAsRead: function(mail_id) {
+	markMailAsRead(mail_id) {
 		// Find the mail object with the specified mail_id
 		let idx = -1;
 		for (let i = 0; i < this._mails.length; i++) {
-			[id, size] = this._mails[i]['id'].get_string();
+			let [id, size] = this._mails[i]['id'].get_string();
 			if (id == mail_id) {
 				idx = i;
 				break;
@@ -166,7 +163,7 @@ const MailnagExtension = new Lang.Class({
 		this._mails.splice(idx, 1);
 		
 		// Notify the Mailnag daemon to mark the mail as read
-		this._proxy.MarkMailAsReadRemote(mail_id, function(result, error) {
+		this._proxy.MarkMailAsReadRemote(mail_id, (result, error) => {
 			if (error) {
 				log("Error: markMailAsReadRemote() failed.");
 			}
@@ -179,15 +176,15 @@ const MailnagExtension = new Lang.Class({
 			if (this._indicator != null)
 				this._indicator.setMails(this._mails);
 		}
-	},
+	}
 	
-	markAllMailsAsRead: function() {
+	markAllMailsAsRead() {
 		// TODO: add a markAllMailsAsRead() method to the DBus interface
 		for (let i = 0; i < this._mails.length; i++) {
-			[id, size] = this._mails[i]['id'].get_string();
+			let [id, size] = this._mails[i]['id'].get_string();
 		
 			// Notify the Mailnag daemon to mark the mail as read
-			this._proxy.MarkMailAsReadRemote(id, function(result, error) {
+			this._proxy.MarkMailAsReadRemote(id, (result, error) => {
 				if (error) {
 					log("Error: markMailAsReadRemote() failed.");
 				}
@@ -202,17 +199,25 @@ const MailnagExtension = new Lang.Class({
 			if (this._indicator != null)
 				this._indicator.setMails(this._mails);
 		}
-	},
+	}
 	
-	checkForMails: function() {
-		this._proxy.CheckForMailsRemote(function(result, error) {
+	checkForMails() {
+		this._proxy.CheckForMailsRemote((result, error) => {
 			if (error) {
 				log("Error: checkForMailsRemote() failed.");
 			}
 		});
-	},
+	}
 	
-	dispose: function() {
+	shutdown() {
+		this._proxy.ShutdownRemote((result, error) => {
+			if (error) {
+				log("Error: shutdownRemote() failed.");
+			}
+		});
+	}
+	
+	dispose() {
 		if (this._onMailsAddedId > -1) {
 			this._proxy.disconnectSignal(this._onMailsAddedId);
 			this._onMailsAddedId = -1;
@@ -225,7 +230,7 @@ const MailnagExtension = new Lang.Class({
 		
 		this._destroyIndicator();
 	}
-});
+};
 
 function aggregateAvatarsAsync(completedCallback) {
 	let aggregator = "aggregate-avatars";
@@ -258,7 +263,7 @@ function aggregateAvatarsAsync(completedCallback) {
 		GLib.close(stderr_fd);
 
 		let childWatch = GLib.child_watch_add(GLib.PRIORITY_DEFAULT, pid, 
-			function(pid, status, requestObject) {
+			(pid, status, requestObject) => {
 				let [out, size] = stdout.read_line(null);
 
 				if (size > 0) {
@@ -278,7 +283,6 @@ function aggregateAvatarsAsync(completedCallback) {
 }
 
 function createExt(s, avatars) {
-	
 	let opts = new Opts.Options();
 	
 	opts.maxVisibleMails 		= s.get_int(MAX_VISIBLE_MAILS_KEY);
@@ -289,7 +293,7 @@ function createExt(s, avatars) {
 	opts.avatarSize				= AVATAR_ICON_SIZE;
 	opts.menuActions			= Opts.ACTION_FLAGS.NONE;
 	
-	for ([k, v] of KeyActionMap) {
+	for (let [k, v] of KeyActionMap) {
 		if (s.get_boolean(k))
 			opts.menuActions |= v;
 	}
@@ -312,13 +316,13 @@ function enable() {
 	// Register changed handler for gsettings.
 	// (Restarts the extension if settings
 	// have been changed, e.g. via prefs.js)
-	settings.connect('changed', function(settings, key) {
+	settings.connect('changed', (settings, key) => {
 		if ((ext != null) && !reloadInProgress) {
 			ext.dispose();
 
 			if ((cachedAvatars == null) && (key == SHOW_AVATARS_KEY) && settings.get_boolean(key)) {
 				reloadInProgress = true;
-				aggregateAvatarsAsync(function(avatars) {
+				aggregateAvatarsAsync((avatars) => {
 						cachedAvatars = avatars;
 						ext = createExt(settings, avatars);
 						reloadInProgress = false;
@@ -332,9 +336,9 @@ function enable() {
 	// Register dbus watch handlers - create the extension 
 	// if the Mailnag daemon is running / remove it if the daemon is gone.
 	watch_id = Gio.DBus.session.watch_name('mailnag.MailnagService', Gio.BusNameWatcherFlags.NONE,
-		function (owner) {
+		(owner) => {
 			if (settings.get_boolean(SHOW_AVATARS_KEY)) {
-				aggregateAvatarsAsync(function(avatars) {
+				aggregateAvatarsAsync((avatars) => {
 						cachedAvatars = avatars;
 						ext = createExt(settings, avatars);
 					});
@@ -342,7 +346,7 @@ function enable() {
 				ext = createExt(settings, {});
 			}
 		},
-		function (owner) {
+		(owner) => {
 			if (ext != null) {
 				ext.dispose();
 				ext = null;
