@@ -1,6 +1,6 @@
 /* Mailnag - GNOME-Shell extension frontend
 *
-* Copyright 2016 Patrick Ulbrich <zulu99@gmx.net>
+* Copyright 2016, 2019, 2020 Patrick Ulbrich <zulu99@gmx.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,24 +18,23 @@
 * MA 02110-1301, USA.
 */
 
-const Lang = imports.lang;
-
-const ACTION_FLAGS = {
+var ACTION_FLAGS = {
 	NONE				: 0,
 	MARK_ALL_AS_READ	: 1,
 	CHECK_FOR_MAIL		: 2,
 	SETTINGS			: 4,
-	ALL					: 7
+	QUIT				: 8,
+	ALL					: 15
 };
 
-const Options = new Lang.Class({
-	Name: 'Options',
-	
-	maxVisibleMails 	: 10,
-	showDates			: true,
-	groupMailsByAccount	: false,
-	removeIndicator		: true,
-	avatars				: {},
-	avatarSize			: 38,
-	menuActions			: ACTION_FLAGS.ALL
-});
+var Options = class {
+	constructor() {	
+		this.maxVisibleMails 		= 10;
+		this.showDates				= true;
+		this.groupMailsByAccount	= false;
+		this.removeIndicator		= true;
+		this.avatars				= {};
+		this.avatarSize				= 38;
+		this.menuActions			= ACTION_FLAGS.ALL;
+	}
+};
